@@ -3,7 +3,7 @@
  *  This include file includes all other public GiNaC headers. */
 
 /*
- *  GiNaC Copyright (C) 1999-2000 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2003 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -23,15 +23,17 @@
 #ifndef __GINAC_H__
 #define __GINAC_H__
 
+#include "version.h"
+
 #include "basic.h"
 
 #include "ex.h"
 #include "normal.h"
 #include "archive.h"
+#include "print.h"
 
 #include "constant.h"
 #include "fail.h"
-#include "idx.h"
 #include "lst.h"
 #include "matrix.h"
 #include "numeric.h"
@@ -39,29 +41,34 @@
 #include "relational.h"
 #include "structure.h"
 #include "symbol.h"
+#include "pseries.h"
+#include "wildcard.h"
+#include "symmetry.h"
 
 #include "expair.h"
 #include "expairseq.h"
 #include "add.h"
 #include "mul.h"
-#include "pseries.h"
 
 #include "exprseq.h"
 #include "function.h"
 #include "ncmul.h"
 
 #include "inifcns.h"
+#include "fderivative.h"
 #include "operators.h"
 
 #ifndef GINAC_BASE_ONLY
+#include "idx.h"
 #include "indexed.h"
-#include "clifford.h"
-#include "coloridx.h"
+#include "tensor.h"
 #include "color.h"
-#include "isospin.h"
-#include "lorentzidx.h"
-#include "simp_lor.h"
-#include "lortensor.h"
+#include "clifford.h"
 #endif // ndef GINAC_BASE_ONLY
 
+#ifdef __MAKECINT__
+#pragma link C++ nestedclass;
+#pragma link C++ nestedtypedef;
+#endif
+
 #endif // ndef __GINAC_H__
