@@ -3,7 +3,7 @@
  *  Interface to GiNaC's initially known functions. */
 
 /*
- *  GiNaC Copyright (C) 1999-2000 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2003 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -26,12 +26,16 @@
 #include "function.h"
 #include "ex.h"
 
-#ifndef NO_NAMESPACE_GINAC
 namespace GiNaC {
-#endif // ndef NO_NAMESPACE_GINAC
 
 /** Absolute value. */
 DECLARE_FUNCTION_1P(abs)
+	
+/** Complex sign. */
+DECLARE_FUNCTION_1P(csgn)
+
+/** Eta function: log(a*b) == log(a) + log(b) + eta(a, b). */
+DECLARE_FUNCTION_2P(eta)
 
 /** Sine. */
 DECLARE_FUNCTION_1P(sin)
@@ -84,36 +88,41 @@ DECLARE_FUNCTION_1P(Li2)
 /** Trilogarithm. */
 DECLARE_FUNCTION_1P(Li3)
 
-// overloading at work: we cannot use the macros
+// overloading at work: we cannot use the macros here
 /** Riemann's Zeta-function. */
 extern const unsigned function_index_zeta1;
-inline function zeta(const ex & p1) {
-    return function(function_index_zeta1, p1);
+template<typename T1>
+inline function zeta(const T1 & p1) {
+	return function(function_index_zeta1, ex(p1));
 }
 /** Derivatives of Riemann's Zeta-function. */
 extern const unsigned function_index_zeta2;
-inline function zeta(const ex & p1, const ex & p2) {
-    return function(function_index_zeta2, p1, p2);
+template<typename T1, typename T2>
+inline function zeta(const T1 & p1, const T2 & p2) {
+	return function(function_index_zeta2, ex(p1), ex(p2));
 }
 
 /** Gamma-function. */
-DECLARE_FUNCTION_1P(gamma)
+DECLARE_FUNCTION_1P(lgamma)
+DECLARE_FUNCTION_1P(tgamma)
 
 /** Beta-function. */
 DECLARE_FUNCTION_2P(beta)
 
-// overloading at work: we cannot use the macros
+// overloading at work: we cannot use the macros here
 /** Psi-function (aka digamma-function). */
 extern const unsigned function_index_psi1;
-inline function psi(const ex & p1) {
-    return function(function_index_psi1, p1);
+template<typename T1>
+inline function psi(const T1 & p1) {
+	return function(function_index_psi1, ex(p1));
 }
 /** Derivatives of Psi-function (aka polygamma-functions). */
 extern const unsigned function_index_psi2;
-inline function psi(const ex & p1, const ex & p2) {
-    return function(function_index_psi2, p1, p2);
+template<typename T1, typename T2>
+inline function psi(const T1 & p1, const T2 & p2) {
+	return function(function_index_psi2, ex(p1), ex(p2));
 }
-    
+	
 /** Factorial function. */
 DECLARE_FUNCTION_1P(factorial)
 
@@ -123,17 +132,14 @@ DECLARE_FUNCTION_2P(binomial)
 /** Order term function (for truncated power series). */
 DECLARE_FUNCTION_1P(Order)
 
-ex lsolve(const ex &eqns, const ex &symbols);
-
-ex ncpower(const ex &basis, unsigned exponent);
+ex lsolve(const ex &eqns, const ex &symbols, unsigned options = determinant_algo::automatic);
 
+/** Check whether a function is the Order (O(n)) function. */
 inline bool is_order_function(const ex & e)
 {
-    return is_ex_the_function(e, Order);
+	return is_ex_the_function(e, Order);
 }
 
-#ifndef NO_NAMESPACE_GINAC
 } // namespace GiNaC
-#endif // ndef NO_NAMESPACE_GINAC
 
 #endif // ndef __GINAC_INIFCNS_H__
