@@ -1,10 +1,12 @@
 /** @file normal.h
  *
- *  Functions for polynomial quotient and remainder, GCD and LCM computation
- *  and square-free factorization. */
+ *  This file defines several functions that work on univariate and
+ *  multivariate polynomials and rational functions.
+ *  These functions include polynomial quotient and remainder, GCD and LCM
+ *  computation, square-free factorization and rational function normalization. */
 
 /*
- *  GiNaC Copyright (C) 1999-2000 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2003 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -24,9 +26,9 @@
 #ifndef __GINAC_NORMAL_H__
 #define __GINAC_NORMAL_H__
 
-#ifndef NO_NAMESPACE_GINAC
+#include "lst.h"
+
 namespace GiNaC {
-#endif // ndef NO_NAMESPACE_GINAC
 
 class ex;
 class symbol;
@@ -37,9 +39,15 @@ extern ex quo(const ex &a, const ex &b, const symbol &x, bool check_args = true)
 // Remainder r(x) of polynomials a(x) and b(x) in Q[x], so that a(x)=b(x)*q(x)+r(x)
 extern ex rem(const ex &a, const ex &b, const symbol &x, bool check_args = true);
 
-// Pseudo-remainder of polynomials a(x) and b(x) in Z[x]
+// Decompose rational function a(x)=N(x)/D(x) into Q(x)+R(x)/D(x) with degree(R, x) < degree(D, x)
+extern ex decomp_rational(const ex &a, const symbol &x);
+
+// Pseudo-remainder of polynomials a(x) and b(x) in Q[x]
 extern ex prem(const ex &a, const ex &b, const symbol &x, bool check_args = true);
 
+// Pseudo-remainder of polynomials a(x) and b(x) in Q[x]
+extern ex sprem(const ex &a, const ex &b, const symbol &x, bool check_args = true);
+
 // Exact polynomial division of a(X) by b(X) in Q[X] (quotient returned in q), returns false when exact division fails
 extern bool divide(const ex &a, const ex &b, ex &q, bool check_args = true);
 
@@ -50,10 +58,14 @@ extern ex gcd(const ex &a, const ex &b, ex *ca = NULL, ex *cb = NULL, bool check
 extern ex lcm(const ex &a, const ex &b, bool check_args = true);
 
 // Square-free factorization of a polynomial a(x)
-extern ex sqrfree(const ex &a, const symbol &x);
+extern ex sqrfree(const ex &a, const lst &l = lst());
+
+// Square-free partial fraction decomposition of a rational function a(x)
+extern ex sqrfree_parfrac(const ex & a, const symbol & x);
+
+// Collect common factors in sums.
+extern ex collect_common_factors(const ex & e);
 
-#ifndef NO_NAMESPACE_GINAC
 } // namespace GiNaC
-#endif // ndef NO_NAMESPACE_GINAC
 
 #endif // ndef __GINAC_NORMAL_H__
